--                        -*- Mode: lua; lua-indent-level: 2; tab-width: 2 -*-
--[[
Baggins_LibFilter

Author: Daphne Pfister <tiiroib02@sneakemail.com>

Adds LibFilter support to Baggins

License

Copyright (c) 2008 Daphne Pfister

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
]]

local Baggins_LibFilter = {} -- pseudo target for events and callbacks
local LibFilter = LibStub("LibFilter-1.0")
local dewdrop = LibStub("Dewdrop-2.0")

local known_filters = {}
local known_settings = {}

local ITEM_TYPE = LibFilter.FilterType.ITEM
local SLOT_TYPE = LibFilter.FilterType.SLOT

local function FixSettings(filter,rule)
	local settings = rule.settings
	if settings then
		rule.no_settings = nil
		if not known_settings[settings] then
			settings = filter:NewSettings(settings)
			rule.settings = settings
			if settings then
				known_settings[settings] = true
			else
				rule.no_settings = true
			end
		end
	elseif not rule.no_settings then
		-- This effectively upgrades old rules provided the filter supports it
		settings = filter:NewSettings(rule)
		if settings then
			rule.settings = settings
			rule.no_settings = nil
			known_settings[settings] = true
		else
			rule.settings = nil
			rule.no_settings = true
		end
	end
	return settings
end

			
local function SlotMatchFast(bag,slot,rule)
	local filtername = rule.type
	local filter = known_filters[filtername]
	if not filter then return end

	return filter:TestIfMatchSplit(FixSettings(filter,rule),SLOT_TYPE,bag,slot)
end

local function SlotMatchSlow(bag,slot,rule)
	local filtername = rule.type
	local filter = known_filters[filtername]
	if not filter then return end

	return filter:TestIfMatch(FixSettings(filter,rule),strjoin(":",SLOT_TYPE,bag,slot))
end

local function ItemMatch(bag,slot,rule)
	local filtername = rule.type
	local filter = known_filters[filtername]
	if not filter then return end

	local link = GetContainerItemLink(bag, slot)
	return filter:TestIfMatch(FixSettings(filter,rule),link)
end

local function GetName(rule)
	local filtername = rule.type
	local filter = known_filters[filtername]
	if not filter then return filtername end

	local name
	if filter.DescribeSettings then
		name = filter:DescribeSettings(FixSettings(filter,rule))
	end

	return name or filter.name or filtername
end

local function CleanRule(filter, rule)
	local settings = rule.settings
	if (settings) then
		known_settings[settings] = nil
		filter:FreeSettings(settings)	
	end
	settings = filter:NewSettings()
	if settings then
		known_settings[settings] = 1
		rule.no_settings = nil
	else
		rule.no_settings = true
	end
	rule.settings = settings
end

local function RuleChanged()
	Baggins:OnRuleChanged()
	Baggins:ForceFullUpdate() -- Guess I need to call this also
end

local function update()
	Baggins:ForceFullUpdate()
end

local function DewDropOptions(rule, level, value)
	local filtername = rule.type
	local filter = known_filters[filtername]
	if not filter then return end

	if not filter.SettingsOptions then return end
	local opttable = filter:SettingsOptions(FixSettings(filter,rule),RuleChanged)
	if opttable then dewdrop:FeedAceOptionsTable(opttable) end
end

local function AddSlotFilter(filtername,filter)
	known_filters[filtername] = filter
	Baggins:AddCustomRule(filtername,
	{
		DisplayName = filter.name or filtername,
		Description = filter.description or filter.name or filtername,
		Matches = filter.TestIfMatchSplit and SlotMatchFast or SlotMatchSlow,
		GetName = GetName,
		CleanRule = function(rule) CleanRule(filter,rule) end,
		DewDropOptions = DewDropOptions
	})
	if filter.RegisterCallback then
		filter.RegisterCallback(Baggins_LibFilter,"FilterUpdate",update)
	end
end

local function AddItemFilter(filtername,filter)
	known_filters[filtername] = filter
	Baggins:AddCustomRule(filtername,
	{
		DisplayName = filter.name or filtername,
		Description = filter.description or filter.name or filtername,
		Matches = ItemMatch,
		GetName = GetName,
		CleanRule = function(rule) CleanRule(filter,rule) end,
		DewDropOptions = DewDropOptions
	})
	if filter.RegisterCallback then
		filter.RegisterCallback(Baggins_LibFilter,"FilterUpdate",update)
	end
end

local function UpdateFilters()
	for filtername,filter in LibFilter:IterateFilters(SLOT_TYPE) do
		if not known_filters[filtername] then
			AddSlotFilter(filtername,filter)
		end
	end

	for filtername,filter in LibFilter:IterateFilters(ITEM_TYPE) do
		if not known_filters[filtername] then
			AddItemFilter(filtername,filter)
		end
	end
end

UpdateFilters()

local events = {}
local myFrame = CreateFrame("Frame","Baggins_LibFilter_Frame")
myFrame:SetScript("OnEvent",function (self, event, ...) events[event](...) end)

function events.ADDON_LOADED()
	UpdateFilters()
end

myFrame:RegisterEvent("ADDON_LOADED");

function events.UPDATE()
	UpdateFilters()
	myFrame:SetScript("OnUpdate",nil)
end

function events.PLAYER_LOGIN()
	myFrame:SetScript("OnUpdate",function (self, ...) events.UPDATE(...) end)
end
myFrame:RegisterEvent("PLAYER_LOGIN");
